@@ -98,6 +98,41 @@ typedef std::pair<std::string, GpgME::Protocol> update_arg_t;
 
 typedef std::pair<std::unique_ptr<LocateArgs>, std::string> import_arg_t;
 
+static std::vector<GpgME::Key>
+filter_chain (const std::vector<GpgME::Key> &input)
+{
+   std::vector<GpgME::Key> leaves;
+
+   std::remove_copy_if(input.begin(), input.end(),
+           std::back_inserter(leaves),
+           [input] (const auto &k)
+            {
+               /* Check if a key has this fingerprint in the
+                * chain ID. Meaning that there is any child of
+                * this certificate. In that case remove it. */
+               for (const auto &c: input)
+               {
+                 if (!c.chainID())
+                  {
+                    continue;
+                  }
+                 if (!k.primaryFingerprint() || !c.primaryFingerprint())
+                  {
+                     STRANGEPOINT;
+                     continue;
+                  }
+                 if (!strcmp (c.chainID(), k.primaryFingerprint()))
+                  {
+                     log_debug ("%s:%s: Filtering %s as non leaf cert",
+                                SRCNAME, __func__, k.primaryFingerprint ());
+                     return true;
+                  }
+               }
+               return false;
+            });
+    return leaves;
+}
+
 static DWORD WINAPI
 do_update (LPVOID arg)
 {
@@ -156,22 +191,31 @@ do_import (LPVOID arg)
   log_debug ("%s:%s importing for: \"%s\" with data \n%s",
              SRCNAME, __func__, anonstr (mbox.c_str ()),
              anonstr (args->second.c_str ()));
-  auto ctx = std::unique_ptr<GpgME::Context> (GpgME::Context::createForProtocol
-                                              (GpgME::OpenPGP));
+
+  // We want to avoid unneccessary copies. The c_str will be valid
+  // until args goes out of scope.
+  const char *keyStr = args->second.c_str ();
+  GpgME::Data data (keyStr, strlen (keyStr), /* copy */ false);
+
+  GpgME::Protocol proto = GpgME::OpenPGP;
+  auto type = data.type();
+  if (type == GpgME::Data::X509Cert)
+    {
+      proto = GpgME::CMS;
+    }
+  data.rewind ();
+
+  auto ctx = GpgME::Context::create(proto);
 
   if (!ctx)
     {
       TRACEPOINT;
       TRETURN 0;
     }
-  // We want to avoid unneccessary copies. The c_str will be valid
-  // until args goes out of scope.
-  const char *keyStr = args->second.c_str ();
-  GpgME::Data data (keyStr, strlen (keyStr), /* copy */ false);
 
-  if (data.type () != GpgME::Data::PGPKey)
+  if (type != GpgME::Data::PGPKey && type != GpgME::Data::X509Cert)
     {
-      log_debug ("%s:%s Data for: %s is not a PGP Key",
+      log_debug ("%s:%s Data for: %s is not a PGP Key or Cert ",
                  SRCNAME, __func__, anonstr (mbox.c_str ()));
       TRETURN 0;
     }
@@ -197,18 +241,24 @@ do_import (LPVOID arg)
 
       update_arg_t * update_args = new update_arg_t;
       update_args->first = std::string (fpr);
-      update_args->second = GpgME::OpenPGP;
+      update_args->second = proto;
 
       // We do it blocking to be sure that when all imports
       // are done they are also part of the keycache.
       do_update ((LPVOID) update_args);
 
-      fingerprints.push_back (fpr);
+      if (std::find(fingerprints.begin(), fingerprints.end(), fpr) ==
+          fingerprints.end())
+        {
+          fingerprints.push_back (fpr);
+        }
       log_debug ("%s:%s Imported: %s from addressbook.",
                  SRCNAME, __func__, anonstr (fpr));
     }
 
-  KeyCache::instance ()->onAddrBookImportJobDone (mbox, fingerprints);
+  KeyCache::instance ()->onAddrBookImportJobDone (mbox,
+                                                  fingerprints,
+                                                  proto);
 
   log_debug ("%s:%s Import job done for: %s",
              SRCNAME, __func__, anonstr (mbox.c_str ()));
@@ -323,6 +373,83 @@ public:
     TRETURN;
   }
 
+  time_t getLastSubkeyCreation(const GpgME::Key &k)
+  {
+    TSTART;
+    time_t ret = 0;
+
+    for (const auto &sub: k.subkeys())
+      {
+        if (sub.isBad())
+          {
+            continue;
+          }
+        if (sub.creationTime() > ret)
+          {
+            ret = sub.creationTime();
+          }
+      }
+    TRETURN ret;
+  }
+
+  GpgME::Key compareSkeys(const GpgME::Key &old, const GpgME::Key &newKey)
+  {
+    TSTART;
+
+    if (newKey.isNull())
+      {
+        return old;
+      }
+    if (old.isNull())
+      {
+        return newKey;
+      }
+
+    if (old.primaryFingerprint() && newKey.primaryFingerprint() &&
+        !strcmp (old.primaryFingerprint(), newKey.primaryFingerprint()))
+      {
+        // Both are the same. Take the newer one.
+        return newKey;
+      }
+
+    if (old.canSign() && !newKey.canSign())
+      {
+        log_debug ("%s:%s Keeping old skey with "
+                   "fpr %s over %s because it can sign.",
+                   SRCNAME, __func__,
+                   anonstr (old.primaryFingerprint()),
+                   anonstr (newKey.primaryFingerprint()));
+        TRETURN old;
+      }
+
+    if (!old.canSign() && newKey.canSign())
+      {
+        log_debug ("%s:%s Using new skey with "
+                   "fpr %s over %s because it can sign.",
+                   SRCNAME, __func__,
+                   anonstr (newKey.primaryFingerprint()),
+                   anonstr (old.primaryFingerprint()));
+        TRETURN newKey;
+      }
+
+    // Both can or can't sign. Use the newest one.
+    if (getLastSubkeyCreation (old) >= getLastSubkeyCreation (newKey))
+      {
+        log_debug ("%s:%s Keeping old skey with "
+                   "fpr %s over %s because it is newer.",
+                   SRCNAME, __func__,
+                   anonstr (old.primaryFingerprint()),
+                   anonstr (newKey.primaryFingerprint()));
+        TRETURN old;
+      }
+    log_debug ("%s:%s Using new skey with "
+               "fpr %s over %s because it is newer.",
+               SRCNAME, __func__,
+               anonstr (newKey.primaryFingerprint()),
+               anonstr (old.primaryFingerprint()));
+    TRETURN newKey;
+  }
+
   void setPgpKeySecret(const std::string &mbox, const GpgME::Key &key,
                        bool insert = true)
   {
@@ -336,7 +463,7 @@ public:
       }
     else
       {
-        it->second = key;
+        it->second = compareSkeys (it->second, key);
       }
     if (insert)
       {
@@ -359,7 +486,7 @@ public:
       }
     else
       {
-        it->second = key;
+        it->second = compareSkeys (it->second, key);
       }
     if (insert)
       {
@@ -369,7 +496,8 @@ public:
     TRETURN;
   }
 
-  std::vector<GpgME::Key> getPGPOverrides (const char *addr)
+  std::vector<GpgME::Key> getOverrides (const char *addr,
+                                        GpgME::Protocol proto = GpgME::OpenPGP)
   {
     TSTART;
     std::vector<GpgME::Key> ret;
@@ -380,9 +508,36 @@ public:
       }
     auto mbox = GpgME::UserID::addrSpecFromString (addr);
 
-    gpgol_lock (&keycache_lock);
-    const auto it = m_addr_book_overrides.find (mbox);
-    if (it == m_addr_book_overrides.end ())
+    gpgol_lock (&import_lock);
+    const auto job_set = (proto == GpgME::OpenPGP ?
+                       &m_pgp_import_jobs : &m_cms_import_jobs);
+    int i = 0;
+    while (job_set->find (mbox) != job_set->end ())
+      {
+        i++;
+        if (i % 100 == 0)
+          {
+            log_debug ("%s:%s Waiting on import for \"%s\"",
+                       SRCNAME, __func__, anonstr (addr));
+          }
+        gpgol_unlock (&import_lock);
+        Sleep (10);
+        gpgol_lock (&import_lock);
+        if (i == 1000)
+          {
+            /* Just to be on the save side */
+            log_error ("%s:%s Waiting on import for \"%s\" "
+                       "failed! Bug!",
+                       SRCNAME, __func__, anonstr (addr));
+            break;
+          }
+      }
+    gpgol_unlock (&import_lock);
+
+    auto override_map = (proto == GpgME::OpenPGP ?
+                         &m_pgp_overrides : &m_cms_overrides);
+    const auto it = override_map->find (mbox);
+    if (it == override_map->end ())
       {
         gpgol_unlock (&keycache_lock);
         TRETURN ret;
@@ -398,7 +553,10 @@ public:
           }
         ret.push_back (key);
       }
-
+    if (proto == GpgME::CMS) {
+        /* Remove root and intermediate ca's */
+        ret = filter_chain (ret);
+    }
     gpgol_unlock (&keycache_lock);
     TRETURN ret;
   }
@@ -523,23 +681,23 @@ public:
       }
     for (const auto &recip: recipients)
       {
-        if (proto == GpgME::OpenPGP)
-          {
-            const auto overrides = getPGPOverrides (recip.c_str ());
+        const auto overrides = getOverrides (recip.c_str (), proto);
 
-            if (!overrides.empty())
-              {
-                ret.insert (ret.end (), overrides.begin (), overrides.end ());
-                log_debug ("%s:%s: Using overides for %s",
-                           SRCNAME, __func__, anonstr (recip.c_str ()));
-                continue;
-              }
+        if (!overrides.empty())
+          {
+            const auto filtered = (proto == GpgME::CMS ? filter_chain(overrides)
+                                                       : overrides);
+            ret.insert (ret.end (), filtered.begin (), filtered.end ());
+            log_debug ("%s:%s: Using overrides for %s",
+                       SRCNAME, __func__, anonstr (recip.c_str ()));
+            continue;
           }
         const auto key = getKey (recip.c_str (), proto);
         if (key.isNull())
           {
-            log_debug ("%s:%s: No key for %s. no internal encryption",
-                       SRCNAME, __func__, anonstr (recip.c_str ()));
+            log_debug ("%s:%s: No key for %s in proto %s. no internal encryption",
+                       SRCNAME, __func__, anonstr (recip.c_str ()),
+                       to_cstr (proto));
             TRETURN std::vector<GpgME::Key>();
           }
 
@@ -854,7 +1012,7 @@ public:
     }
 
   void importFromAddrBook (const std::string &mbox, const char *data,
-                           Mail *mail)
+                           Mail *mail, GpgME::Protocol proto)
     {
       TSTART;
       if (!data || mbox.empty() || !mail)
@@ -870,13 +1028,16 @@ public:
           TRETURN;
         }
       gpgol_lock (&import_lock);
-      if (m_import_jobs.find (mbox) != m_import_jobs.end ())
+      auto job_set = (proto == GpgME::OpenPGP ?
+                       &m_pgp_import_jobs : &m_cms_import_jobs);
+      if (job_set->find (mbox) != job_set->end ())
         {
-          log_debug ("%s:%s import for \"%s\" already in progress.",
-                     SRCNAME, __func__, anonstr (mbox.c_str ()));
+          log_debug ("%s:%s import for \"%s\" %s already in progress.",
+                     SRCNAME, __func__, anonstr (mbox.c_str ()),
+                     to_cstr (proto));
           gpgol_unlock (&import_lock);
         }
-      m_import_jobs.insert (mbox);
+      job_set->insert (mbox);
       gpgol_unlock (&import_lock);
 
       import_arg_t * args = new import_arg_t;
@@ -890,32 +1051,38 @@ public:
     }
 
   void onAddrBookImportJobDone (const std::string &mbox,
-                                const std::vector<std::string> &result_fprs)
+                                const std::vector<std::string> &result_fprs,
+                                GpgME::Protocol proto)
     {
       TSTART;
       gpgol_lock (&keycache_lock);
-      auto it = m_addr_book_overrides.find (mbox);
-      if (it != m_addr_book_overrides.end ())
+      auto override_map = (proto == GpgME::OpenPGP ?
+                           &m_pgp_overrides : &m_cms_overrides);
+      auto job_set = (proto == GpgME::OpenPGP ?
+                       &m_pgp_import_jobs : &m_cms_import_jobs);
+
+      auto it = override_map->find (mbox);
+      if (it != override_map->end ())
         {
           it->second = result_fprs;
         }
       else
         {
-          m_addr_book_overrides.insert (
-                std::make_pair (mbox, result_fprs));
+          override_map->insert (std::make_pair (mbox, result_fprs));
         }
       gpgol_unlock (&keycache_lock);
       gpgol_lock (&import_lock);
-      const auto job_it = m_import_jobs.find(mbox);
+      const auto job_it = job_set->find(mbox);
 
-      if (job_it == m_import_jobs.end())
+      if (job_it == job_set->end())
         {
-          log_error ("%s:%s import for \"%s\" already finished.",
-                     SRCNAME, __func__, anonstr (mbox.c_str ()));
+          log_error ("%s:%s import for \"%s\" %s already finished.",
+                     SRCNAME, __func__, anonstr (mbox.c_str ()),
+                     to_cstr (proto));
           gpgol_unlock (&import_lock);
           TRETURN;
         }
-      m_import_jobs.erase (job_it);
+      job_set->erase (job_it);
       gpgol_unlock (&import_lock);
       TRETURN;
     }
@@ -939,10 +1106,13 @@ public:
   std::unordered_map<std::string, GpgME::Key> m_fpr_map;
   std::unordered_map<std::string, std::string> m_sub_fpr_map;
   std::unordered_map<std::string, std::vector<std::string> >
-    m_addr_book_overrides;
+    m_pgp_overrides;
+  std::unordered_map<std::string, std::vector<std::string> >
+    m_cms_overrides;
   std::vector<GpgME::Key> m_ultimate_keys;
   std::set<std::string> m_update_jobs;
-  std::set<std::string> m_import_jobs;
+  std::set<std::string> m_pgp_import_jobs;
+  std::set<std::string> m_cms_import_jobs;
 };
 
 KeyCache::KeyCache():
@@ -1358,10 +1528,13 @@ KeyCache::isMailResolvable(Mail *mail)
     }
 
 
+  GpgME::Key sigKey = getSigningKey (sender.c_str(), GpgME::OpenPGP);
   std::vector<GpgME::Key> encKeys = getEncryptionKeys (recps,
                                                        GpgME::OpenPGP);
 
-  if (!encKeys.empty())
+  /* If S/MIME is prefrerred we only toggle auto encrypt for PGP if
+     we both have a signing key and encryption keys. */
+  if (!encKeys.empty() && (!opt.prefer_smime || !sigKey.isNull()))
     {
       TRETURN true;
     }
@@ -1374,8 +1547,8 @@ KeyCache::isMailResolvable(Mail *mail)
   /* Check S/MIME instead here we need to include the sender
      as we can't just generate a key. */
   recps.push_back (sender);
-  GpgME::Key sigKey= getSigningKey (sender.c_str(), GpgME::CMS);
   encKeys = getEncryptionKeys (recps, GpgME::CMS);
+  sigKey = getSigningKey (sender.c_str(), GpgME::CMS);
 
   TRETURN !encKeys.empty() && !sigKey.isNull();
 }
@@ -1400,22 +1573,23 @@ KeyCache::onUpdateJobDone (const char *fpr, const GpgME::Key &key)
 
 void
 KeyCache::importFromAddrBook (const std::string &mbox, const char *key_data,
-                              Mail *mail) const
+                              Mail *mail, GpgME::Protocol proto) const
 {
-  return d->importFromAddrBook (mbox, key_data, mail);
+  return d->importFromAddrBook (mbox, key_data, mail, proto);
 }
 
 void
 KeyCache::onAddrBookImportJobDone (const std::string &mbox,
-                                   const std::vector<std::string> &result_fprs)
+                                   const std::vector<std::string> &result_fprs,
+                                   GpgME::Protocol proto)
 {
-  return d->onAddrBookImportJobDone (mbox, result_fprs);
+  return d->onAddrBookImportJobDone (mbox, result_fprs, proto);
 }
 
 std::vector<GpgME::Key>
-KeyCache::getOverrides (const std::string &mbox)
+KeyCache::getOverrides (const std::string &mbox, GpgME::Protocol proto)
 {
-  return d->getPGPOverrides (mbox.c_str ());
+  return d->getOverrides (mbox.c_str (), proto);
 }
 
 void
