@@ -281,10 +281,14 @@ public:
    * store the sender address for later events that do not allow us to
    * access the OOM but enable us to work with the underlying MAPI structure.
    *
+   * for_encryption can be used to override that data should be collected
+   * for encryption. Otherwise it depends on "isCryptoMail" to decide which
+   * data should be collected.
+   *
    * It also updated the is_html_alternative value.
    *
    * @returns 0 on success */
-  int updateOOMData_o ();
+  int updateOOMData_o (bool for_encryption = false);
 
   /** @brief get sender SMTP address (UTF-8 encoded).
    *
@@ -484,12 +488,16 @@ public:
   bool isAsyncCryptDisabled () { return m_async_crypt_disabled; }
 
   /** Check through OOM if the current mail is an inline
-    response.
+    response. Meaning editable in the message list.
+  */
+  bool isActiveInlineResponse_o ();
+
+  /* Check if we can't do async crypto. E.g. for inline responses.
 
     Caches the state which can then be queried through
-    async_crypt_disabled
+    isAsyncCryptDisabled;
   */
-  bool check_inline_response ();
+  bool checkSyncCrypto_o ();
 
   /** Get the window for the mail. Caution! This is only
     really valid in the time that the window is disabled.
@@ -604,6 +612,25 @@ public:
   /* Remove encryption permanently. */
   void decryptPermanently_o ();
 
+  /* Prepare for encrypt / sign. Updates data. */
+  void prepareCrypto_o ();
+
+  /* State variable to check if we are about to encrypt a draft. */
+  void setIsDraftEncrypt (bool value) { m_is_draft_encrypt = value; }
+  bool isDraftEncrypt () { return m_is_draft_encrypt; }
+
+  /* Was this mail decrypted without error. Also returns true
+     if the mail was not encrypted. */
+  bool decryptedSuccessfully () const;
+
+  /* The mail should be decrypted again after the next
+   * encryption. So that we can save it for example and
+   * still work on it. */
+  void setDecryptAgain (bool value) { m_decrypt_again = value; }
+  bool isDecryptAgain () const { return m_decrypt_again; }
+
+  msgtype_t msgtype () const { return m_type; }
+
 private:
   void updateCategories_o ();
   void updateSigstate ();
@@ -658,5 +685,8 @@ private:
   std::string m_store_id; /* Store id for categories */
   std::string m_verify_category; /* The category string for the verify result */
   bool m_is_junk; /* Mail is in the junk folder */
+  bool m_is_draft_encrypt; /* Mail is a draft that should be encrypted */
+  bool m_decrypt_again; /* Mail should be decrypted again if it sees
+                           another beforeread */
 };
 #endif // MAIL_H
