@@ -1012,7 +1012,9 @@ count_usable_attachments (mapi_attach_item_t *table)
 /* Write out all attachments from TABLE separated by BOUNDARY to SINK.
    This function needs to be syncronized with count_usable_attachments.
    If only_related is 1 only include attachments for multipart/related they
-   are excluded otherwise. */
+   are excluded otherwise.
+   If only_related is 2 all attachments are included regardless of
+   content-id. */
 static int
 write_attachments (sink_t sink,
                    LPMESSAGE message, mapi_attach_item_t *table,
@@ -1029,7 +1031,7 @@ write_attachments (sink_t sink,
         if (table[idx].attach_type == ATTACHTYPE_UNKNOWN
             && table[idx].method == ATTACH_BY_VALUE)
           {
-            if (only_related && !table[idx].content_id)
+            if (only_related == 1 && !table[idx].content_id)
               {
                 continue;
               }
@@ -1052,44 +1054,48 @@ write_attachments (sink_t sink,
               }
             xfree (buffer);
           }
-        else
-          {
-            if (!only_related && !warning_shown
+        else if (!only_related && !warning_shown
                 && table[idx].attach_type == ATTACHTYPE_UNKNOWN
                 && (table[idx].method == ATTACH_OLE
                     || table[idx].method == ATTACH_EMBEDDED_MSG))
+          {
+            char *fmt;
+            log_debug ("%s:%s: detected OLE attachment. Showing warning.",
+                       SRCNAME, __func__);
+            gpgrt_asprintf (&fmt, _("The attachment '%s' is an Outlook item "
+                                    "which is currently unsupported in crypto mails."),
+                            table[idx].filename ?
+                            table[idx].filename : _("Unknown"));
+            std::string msg = fmt;
+            msg += "\n\n";
+            xfree (fmt);
+
+            gpgrt_asprintf (&fmt, _("Please encrypt '%s' with Kleopatra "
+                                    "and attach it as a file."),
+                            table[idx].filename ?
+                            table[idx].filename : _("Unknown"));
+            msg += fmt;
+            xfree (fmt);
+
+            msg += "\n\n";
+            msg += _("Send anyway?");
+            warning_shown = true;
+
+            if (gpgol_message_box (get_active_hwnd (),
+                                   msg.c_str (),
+                                   _("Sorry, that's not possible, yet"),
+                                   MB_APPLMODAL | MB_YESNO) == IDNO)
               {
-                char *fmt;
-                log_debug ("%s:%s: detected OLE attachment. Showing warning.",
-                           SRCNAME, __func__);
-                gpgrt_asprintf (&fmt, _("The attachment '%s' is an Outlook item "
-                                        "which is currently unsupported in crypto mails."),
-                                table[idx].filename ?
-                                table[idx].filename : _("Unknown"));
-                std::string msg = fmt;
-                msg += "\n\n";
-                xfree (fmt);
-
-                gpgrt_asprintf (&fmt, _("Please encrypt '%s' with Kleopatra "
-                                        "and attach it as a file."),
-                                table[idx].filename ?
-                                table[idx].filename : _("Unknown"));
-                msg += fmt;
-                xfree (fmt);
-
-                msg += "\n\n";
-                msg += _("Send anyway?");
-                warning_shown = true;
-
-                if (gpgol_message_box (get_active_hwnd (),
-                                       msg.c_str (),
-                                       _("Sorry, that's not possible, yet"),
-                                       MB_APPLMODAL | MB_YESNO) == IDNO)
-                  {
-                    return -1;
-                  }
+                return -1;
               }
           }
+        else
+          {
+            log_debug ("%s:%s: Skipping unknown attachment at idx: %d type: %d"
+                       " with method: %d",
+                       SRCNAME, __func__, idx, table[idx].attach_type,
+                       table[idx].method);
+          }
       }
   return 0;
 }
@@ -1212,7 +1218,8 @@ cancel_mapi_attachment (LPATTACH *attach, sink_t sink)
 /* Do the final processing for a message. */
 int
 finalize_message (LPMESSAGE message, mapi_attach_item_t *att_table,
-                  protocol_t protocol, int encrypt, bool is_inline)
+                  protocol_t protocol, int encrypt, bool is_inline,
+                  bool is_draft, int exchange_major_version)
 {
   HRESULT hr = 0;
   SPropValue prop;
@@ -1220,7 +1227,28 @@ finalize_message (LPMESSAGE message, mapi_attach_item_t *att_table,
 
   /* Set the message class.  */
   prop.ulPropTag = PR_MESSAGE_CLASS_A;
-  if (encrypt)
+  if (protocol == PROTOCOL_SMIME)
+    {
+      /* When sending over exchange to the same server the recipient
+         might see the message class we set here. So for S/MIME
+         we keep the original. This makes the sent folder icon
+         not immediately showing the GpgOL icon but gives other
+         clients that do not have GpgOL installed a better chance
+         to handle the mail. */
+      if (encrypt && /* exchange_major_version >= 15*/ 0)
+        {
+          (void) exchange_major_version;
+          /* This only appears to work with later exchange versions and
+          caused other Problems with incompatibilities. MultipartSigned
+          seemd to work better, even if it now has the wrong icon. */
+          prop.Value.lpszA = xstrdup ("IPM.Note.SMIME");
+        }
+      else
+        {
+          prop.Value.lpszA = xstrdup ("IPM.Note.SMIME.MultipartSigned");
+        }
+    }
+  else if (encrypt)
     {
       prop.Value.lpszA = xstrdup ("IPM.Note.InfoPathForm.GpgOL.SMIME.MultipartSigned");
     }
@@ -1298,8 +1326,16 @@ finalize_message (LPMESSAGE message, mapi_attach_item_t *att_table,
     }
 
   /* Remove the draft info so that we don't leak the information on
-     whether the message has been signed etc.  */
-  mapi_set_gpgol_draft_info (message, NULL);
+     whether the message has been signed etc. when we send it.
+     If it is a draft we are encrypting we want to keep them.
+
+     To avoid confusion: draft_info for us means the state of
+     the secure toggle button.
+     */
+  if (!is_draft)
+    {
+      mapi_set_gpgol_draft_info (message, NULL);
+    }
 
   if (mapi_save_changes (message, KEEP_OPEN_READWRITE|FORCE_SAVE))
     {
@@ -1499,10 +1535,22 @@ add_body_and_attachments (sink_t sink, LPMESSAGE message,
         }
     }
 
-  /* Now write the other attachments */
+  /* Now write the other attachments.
+
+     If we are multipart related the related attachments were already
+     written above. If we are not related we pass 2 to the write_attachements
+     function to force that even attachments with a content id are written
+     out.
+
+     This happens for example when forwarding a plain text mail with
+     attachments.
+     */
   if (!rc && n_att_usable)
-    rc = write_attachments (sink, message, att_table,
-                            *outer_boundary? outer_boundary : NULL, 0);
+    {
+      rc = write_attachments (sink, message, att_table,
+                              *outer_boundary? outer_boundary : NULL,
+                              related ? 0 : 2);
+    }
   if (rc)
     {
       return rc;
