@@ -64,8 +64,8 @@ gpgol_openProperty (LPMAPIPROP obj, ULONG ulPropTag, LPCIID lpiid,
                                    lppUnk);
   if (ret)
     {
-      log_debug ("%s:%s: OpenProperty failed hr=%#lx",
-                 SRCNAME, __func__, ret);
+      log_debug ("%s:%s: OpenProperty failed hr=%#lx %s",
+                 SRCNAME, __func__, ret, mapi_err_to_string (ret));
     }
   else if ((opt.enable_debug & DBG_MEMORY) && *lppUnk)
     {
@@ -240,7 +240,8 @@ get_oom_object (LPDISPATCH pStart, const char *fullname)
         {
           log_error ("%s:%s Object does not support IDispatch",
                      SRCNAME, __func__);
-          gpgol_release (pObj);
+          if (pObj != pStart)
+            gpgol_release (pObj);
           TRETURN NULL;
         }
       /* Confirmed through testing that the retval needs a release */
@@ -2593,6 +2594,9 @@ get_ex_major_version_for_addr (const char *mbox)
     {
       TRETURN -1;
     }
+
+  log_debug ("%s:%s: Detected exchange major version: %s",
+             SRCNAME, __func__, version_str);
   long int version = strtol (version_str, nullptr, 10);
   xfree (version_str);
 
@@ -2917,8 +2921,32 @@ release_disp (LPDISPATCH obj)
   TRETURN;
 }
 
-bool
-is_junk_mail (LPDISPATCH mailitem)
+enum FolderID
+{
+  olFolderCalendar = 9,
+  olFolderConflicts = 19,
+  olFolderContacts = 10,
+  olFolderDeletedItems = 3,
+  olFolderDrafts = 16,
+  olFolderInbox = 6,
+  olFolderJournal = 11,
+  olFolderJunk = 23,
+  olFolderLocalFailures = 21,
+  olFolderManagedEmail = 29,
+  olFolderNotes = 12,
+  olFolderOutbox = 4,
+  olFolderSentMail = 5,
+  olFolderServerFailures = 22,
+  olFolderSuggestedContacts = 30,
+  olFolderSyncIssues = 20,
+  olFolderTasks = 13,
+  olFolderToDo = 28,
+  olPublicFoldersAllPublicFolders = 18,
+  olFolderRssFeeds = 25,
+};
+
+static bool
+is_mail_in_folder (LPDISPATCH mailitem, int folder)
 {
   TSTART;
   if (!mailitem)
@@ -2927,18 +2955,21 @@ is_junk_mail (LPDISPATCH mailitem)
       TRETURN false;
     }
 
-  auto mapi_namespace = MAKE_SHARED (get_oom_object (mailitem, "Session"));
+  auto store = MAKE_SHARED (get_oom_object (mailitem, "Parent.Store"));
 
-  if (!mapi_namespace)
+  if (!store)
     {
-      STRANGEPOINT;
+      log_debug ("%s:%s: Mail has no parent folder. Probably unsafed",
+                 SRCNAME, __func__);
       TRETURN false;
     }
 
-  auto spam_folder = MAKE_SHARED (get_oom_object (mapi_namespace.get(),
-                                                  "GetDefaultFolder(23)"));
+  std::string tmp = std::string("GetDefaultFolder(") + std::to_string (folder) +
+                    std::string(")");
+  auto target_folder = MAKE_SHARED (get_oom_object (store.get(),
+                                                    tmp.c_str()));
 
-  if (!spam_folder)
+  if (!target_folder)
     {
       STRANGEPOINT;
       TRETURN false;
@@ -2952,8 +2983,8 @@ is_junk_mail (LPDISPATCH mailitem)
       TRETURN false;
     }
 
-  char *spam_id = get_oom_string (spam_folder.get(), "entryID");
-  if (!spam_id)
+  char *target_id = get_oom_string (target_folder.get(), "entryID");
+  if (!target_id)
     {
       STRANGEPOINT;
       TRETURN false;
@@ -2962,14 +2993,27 @@ is_junk_mail (LPDISPATCH mailitem)
   if (!folder_id)
     {
       STRANGEPOINT;
-      free (spam_id);
+      free (target_id);
       TRETURN false;
     }
 
-  bool ret = !strcmp (spam_id, folder_id);
+  bool ret = !strcmp (target_id, folder_id);
 
-  free (spam_id);
+  free (target_id);
   free (folder_id);
-
   TRETURN ret;
 }
+
+bool
+is_junk_mail (LPDISPATCH mailitem)
+{
+  TSTART;
+  TRETURN is_mail_in_folder (mailitem, FolderID::olFolderJunk);
+}
+
+bool
+is_draft_mail (LPDISPATCH mailitem)
+{
+  TSTART;
+  TRETURN is_mail_in_folder (mailitem, FolderID::olFolderDrafts);
+}
