@@ -285,6 +285,28 @@ ParseController::setSender(const std::string &sender)
   TRETURN;
 }
 
+static void
+handle_autocrypt_info (const autocrypt_s &info)
+{
+  TSTART;
+  if (info.pref.size() && info.pref != "mutual")
+    {
+      log_debug ("%s:%s: Autocrypt preference is %s which is unhandled.",
+                 SRCNAME, __func__, info.pref.c_str());
+      TRETURN;
+    }
+
+#ifndef BUILD_TESTS
+  if (!KeyCache::import_pgp_key_data (info.data))
+    {
+      log_error ("%s:%s: Failed to import", SRCNAME, __func__);
+    }
+#endif
+  /* No need to free b64decoded as gpgme_data_release handles it */
+
+  TRETURN;
+}
+
 static bool
 is_valid_chksum(const GpgME::Signature &sig)
 {
@@ -328,6 +350,11 @@ ParseController::parse()
 
   auto inputType = input.type ();
 
+  if (m_autocrypt_info.exists)
+    {
+      handle_autocrypt_info (m_autocrypt_info);
+    }
+
   if (inputType == Data::Type::PGPSigned)
     {
       verify = true;
@@ -546,6 +573,20 @@ ParseController::parse()
       m_block_html = true;
     }
 
+  /* Import any application/pgp-keys attachments if the option is set. */
+  if (opt.autoimport)
+    {
+      for (const auto &attach: get_attachments())
+        {
+          if (attach->get_content_type () == "application/pgp-keys")
+            {
+#ifndef BUILD_TESTS
+              KeyCache::import_pgp_key_data (attach->get_data());
+#endif
+            }
+        }
+    }
+
   if (opt.enable_debug & DBG_DATA)
     {
       std::stringstream ss;
@@ -655,3 +696,14 @@ ParseController::get_attachments() const
       TRETURN std::vector<std::shared_ptr<Attachment> >();
     }
 }
+
+std::string
+ParseController::get_internal_subject() const
+{
+  TSTART;
+  if (m_outputprovider)
+    {
+      TRETURN m_outputprovider->get_internal_subject ();
+    }
+  TRETURN std::string();
+}
